package inf112.skeleton.app;

import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import javafx.animation.AnimationTimer;
import javafx.application.Application;
import javafx.event.EventHandler;
import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.effect.Bloom;
import javafx.scene.effect.BoxBlur;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;
import javafx.scene.paint.Color;
import javafx.scene.paint.CycleMethod;
import javafx.scene.paint.Paint;
import javafx.scene.paint.RadialGradient;
import javafx.scene.paint.Stop;
import javafx.scene.shape.ArcType;
import javafx.stage.Stage;

public class Game extends Application {
    private AnimationTimer timer;
    private Canvas canvas;
    private long nanosPerStep = 1000_000_000L / 60L;
    private long timeBudget = nanosPerStep;
    private long lastUpdateTime = 0L;;

    private List<Wall> ListWalls = new ArrayList<>();
    private Player player;
    public Wall wall;


    public static void startIt(String[] args) {
        launch(args);
    }

    @Override
    public void start(Stage stage) {
        double width = 500;
        double height = 500;
        Group root = new Group();
        Scene scene = new Scene(root, width, height, Color.LIGHTBLUE);
        stage.setScene(scene);

        canvas = new Canvas(width, height);
        canvas.widthProperty().bind(scene.widthProperty());
        canvas.heightProperty().bind(scene.heightProperty());

        setup();
        KeyHandler keyH = new KeyHandler(player);
        scene.setOnKeyPressed(keyH::keyPressed);
        scene.setOnKeyReleased(keyH::keyReleased);


        timer = new AnimationTimer() {

            @Override
            public void handle(long now) {
                // System.out.println("Elapsed: " + (now -
                // lastUpdateTime)/(double)millisPerStep);
                if (lastUpdateTime > 0) {
                    timeBudget = Math.min(timeBudget + (now - lastUpdateTime), 10 * nanosPerStep);
                }
                lastUpdateTime = now;

                while (timeBudget >= nanosPerStep) {
                    // System.out.println("Budget: " + timeBudget);
                    timeBudget = timeBudget - nanosPerStep;
                    step();
                }
                draw();
            }

        };
        root.getChildren().add(canvas);

        timer.start();
        stage.show();
    }

    private void setup() {
        player = new Player();

        ListWalls.add(new Wall(250, 400, 150, 20, Color.GREEN));
        ListWalls.add(new Wall(250, 490, 500, 20, Color.GREEN));
        //ListWalls.add(new Wall(0, 250, 20, 500, Color.GREEN));
   }

    protected void step() {
        //System.out.println(player.getX());
        //System.out.println(player.getY());

        player.canMoveLeft = true;
        player.canMoveRight = true;
        player.canMoveDown = true;
        player.canMoveUp = true;
        for (Wall wall : ListWalls) {
            player.checkCollision(wall);
        }

        player.Move();
    }


    protected void draw() {
        GraphicsContext context = canvas.getGraphicsContext2D();
        context.clearRect(0, 0, canvas.getWidth(), canvas.getHeight());

        player.drawPlayer(context);

        for (Wall wall : ListWalls) {
            int walls_w = wall.getWidth();
            int walls_h = wall.getHeight();
            double walls_xPos = wall.getX() - walls_w / 2.0;
            double walls_yPos = wall.getY() - walls_h  / 2.0;
            context.save();
            Paint c = wall.getColor();
            context.setFill(c);
            context.fillRect(walls_xPos, walls_yPos, walls_w, walls_h);
            context.restore();
        }
    }
}


